@@ -17,12 +17,12 @@ import re
 
 import netaddr
 from neutron.common import ipv6_utils
-from neutron.db import api as db_api
-from neutron.db import common_db_mixin as base_db
 from neutron_lib.callbacks import events
 from neutron_lib.callbacks import registry
 from neutron_lib.callbacks import resources
 from neutron_lib import constants as n_const
+from neutron_lib.db import api as db_api
+from neutron_lib.db import model_query
 from neutron_lib import exceptions as n_exc
 from neutron_lib.plugins import constants as pg_const
 from neutron_lib.plugins import directory
@@ -33,6 +33,7 @@ from oslo_utils import uuidutils
 from sqlalchemy import orm
 from sqlalchemy.orm import exc
 from sqlalchemy.orm import lazyload
+from sqlalchemy.orm import subqueryload
 
 from neutron_lbaas._i18n import _
 from neutron_lbaas import agent_scheduler
@@ -48,8 +49,7 @@ from neutron_lbaas.services.loadbalancer import data_models
 LOG = logging.getLogger(__name__)
 
 
-class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
-                             agent_scheduler.LbaasAgentSchedulerDbMixin):
+class LoadBalancerPluginDbv2(agent_scheduler.LbaasAgentSchedulerDbMixin):
     """Wraps loadbalancer with SQLAlchemy models.
 
     A class that wraps the implementation of the Neutron loadbalancer
@@ -67,13 +67,13 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
                 # To lock the instance for update, return a single
                 # instance, instead of an instance with LEFT OUTER
                 # JOINs that do not work in PostgreSQL
-                query = self._model_query(context, model).options(
+                query = model_query.query_with_hooks(context, model).options(
                     lazyload('*')
                 ).filter(
                     model.id == id).with_lockmode('update')
                 resource = query.one()
             else:
-                resource = self._get_by_id(context, model, id)
+                resource = model_query.get_by_id(context, model, id)
         except exc.NoResultFound:
             with excutils.save_and_reraise_exception(reraise=False) as ctx:
                 if issubclass(model, (models.LoadBalancer, models.Listener,
@@ -88,14 +88,16 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
 
     def _resource_exists(self, context, model, id):
         try:
-            self._get_by_id(context, model, id)
+            model_query.get_by_id(context, model, id)
         except exc.NoResultFound:
             return False
         return True
 
-    def _get_resources(self, context, model, filters=None):
-        query = self._get_collection_query(context, model,
-                                           filters=filters)
+    def _get_resources(self, context, model, filters=None, options=None):
+        query = model_query.get_collection_query(context, model,
+                                                 filters=filters)
+        if options:
+            query = query.options(options)
         return [model_instance for model_instance in query]
 
     def _create_port_choose_fixed_ip(self, fixed_ips):
@@ -249,7 +251,7 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
         with context.session.begin(subtransactions=True):
             if issubclass(model, models.LoadBalancer):
                 try:
-                    model_db = (self._model_query(context, model).
+                    model_db = (model_query.query_with_hooks(context, model).
                                 filter(model.id == id).
                                 options(orm.noload('vip_port')).
                                 one())
@@ -381,7 +383,7 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
             return
         if port_db['device_owner'] == n_const.DEVICE_OWNER_LOADBALANCERV2:
             filters = {'vip_port_id': [port_id]}
-            if len(self.get_loadbalancers(context, filters=filters)) > 0:
+            if len(self.get_loadbalancer_ids(context, filters=filters)) > 0:
                 reason = _('has device owner %s') % port_db['device_owner']
                 raise n_exc.ServicePortInUse(port_id=port_db['id'],
                                              reason=reason)
@@ -391,16 +393,41 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
             _prevent_lbaasv2_port_delete_callback, resources.PORT,
             events.BEFORE_DELETE)
 
+    def get_loadbalancer_ids(self, context, filters=None):
+        lb_dbs = self._get_resources(context, models.LoadBalancer,
+                                     filters=filters)
+        return [lb_db.id
+                for lb_db in lb_dbs]
+
     def get_loadbalancers(self, context, filters=None):
         lb_dbs = self._get_resources(context, models.LoadBalancer,
                                      filters=filters)
         return [data_models.LoadBalancer.from_sqlalchemy_model(lb_db)
                 for lb_db in lb_dbs]
 
+    def get_loadbalancers_as_api_dict(self, context, filters=None):
+        options = (
+            subqueryload(models.LoadBalancer.listeners),
+            subqueryload(models.LoadBalancer.pools),
+            subqueryload(models.LoadBalancer.provider)
+        )
+        lb_dbs = self._get_resources(context, models.LoadBalancer,
+                                     filters=filters, options=options)
+        return [lb_db.to_api_dict
+                for lb_db in lb_dbs]
+
+    def get_provider_names_used_in_loadbalancers(self, context):
+        lb_dbs = self._get_resources(context, models.LoadBalancer)
+        return [lb_db.provider.provider_name for lb_db in lb_dbs]
+
     def get_loadbalancer(self, context, id):
         lb_db = self._get_resource(context, models.LoadBalancer, id)
         return data_models.LoadBalancer.from_sqlalchemy_model(lb_db)
 
+    def get_loadbalancer_as_api_dict(self, context, id):
+        lb_db = self._get_resource(context, models.LoadBalancer, id)
+        return lb_db.to_api_dict
+
     def _validate_listener_data(self, context, listener):
         pool_id = listener.get('default_pool_id')
         lb_id = listener.get('loadbalancer_id')
@@ -444,8 +471,8 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
             if pool.loadbalancer_id != listener.loadbalancer_id:
                 raise sharedpools.ListenerAndPoolMustBeOnSameLoadbalancer()
 
-        if (l7policy['action'] == lb_const.L7_POLICY_ACTION_REDIRECT_TO_URL
-            and 'redirect_url' not in l7policy):
+        if (l7policy['action'] == lb_const.L7_POLICY_ACTION_REDIRECT_TO_URL and
+            'redirect_url' not in l7policy):
             raise l7.L7PolicyRedirectUrlMissing()
 
     def _validate_l7rule_data(self, context, rule):
@@ -574,10 +601,25 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
         return [data_models.Listener.from_sqlalchemy_model(listener_db)
                 for listener_db in listener_dbs]
 
+    def get_listeners_as_api_dict(self, context, filters=None):
+        options = (
+            subqueryload(models.Listener.sni_containers),
+            subqueryload(models.Listener.loadbalancer),
+            subqueryload(models.Listener.l7_policies)
+        )
+        listener_dbs = self._get_resources(context, models.Listener,
+                                           filters=filters, options=options)
+        return [listener_db.to_api_dict
+                for listener_db in listener_dbs]
+
     def get_listener(self, context, id):
         listener_db = self._get_resource(context, models.Listener, id)
         return data_models.Listener.from_sqlalchemy_model(listener_db)
 
+    def get_listener_as_api_dict(self, context, id):
+        listener_db = self._get_resource(context, models.Listener, id)
+        return listener_db.to_api_dict
+
     def _create_session_persistence_db(self, session_info, pool_id):
         session_info['pool_id'] = pool_id
         return models.SessionPersistenceV2(**session_info)
@@ -677,8 +719,9 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
                                      {'default_pool_id': None})
             for l in pool_db.loadbalancer.listeners:
                 for p in l.l7_policies:
-                    if (p.action == lb_const.L7_POLICY_ACTION_REDIRECT_TO_POOL
-                        and p.redirect_pool_id == id):
+                    if (p.action ==
+                        lb_const.L7_POLICY_ACTION_REDIRECT_TO_POOL and
+                        p.redirect_pool_id == id):
                         self.update_l7policy(
                             context, p.id,
                             {'redirect_pool_id': None,
@@ -690,10 +733,27 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
         return [data_models.Pool.from_sqlalchemy_model(pool_db)
                 for pool_db in pool_dbs]
 
+    def get_pools_as_api_dict(self, context, filters=None):
+        options = (
+            subqueryload(models.PoolV2.members),
+            subqueryload(models.PoolV2.listeners),
+            subqueryload(models.PoolV2.l7_policies),
+            subqueryload(models.PoolV2.loadbalancer),
+            subqueryload(models.PoolV2.session_persistence)
+        )
+        pool_dbs = self._get_resources(context, models.PoolV2,
+            filters=filters, options=options)
+        return [pool_db.to_api_dict
+                for pool_db in pool_dbs]
+
     def get_pool(self, context, id):
         pool_db = self._get_resource(context, models.PoolV2, id)
         return data_models.Pool.from_sqlalchemy_model(pool_db)
 
+    def get_pool_as_api_dict(self, context, id):
+        pool_db = self._get_resource(context, models.PoolV2, id)
+        return pool_db.to_api_dict
+
     def create_pool_member(self, context, member, pool_id):
         try:
             with context.session.begin(subtransactions=True):
@@ -729,10 +789,21 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
         return [data_models.Member.from_sqlalchemy_model(member_db)
                 for member_db in member_dbs]
 
+    def get_pool_members_as_api_dict(self, context, filters=None):
+        filters = filters or {}
+        member_dbs = self._get_resources(context, models.MemberV2,
+                                         filters=filters)
+        return [member_db.to_api_dict
+                for member_db in member_dbs]
+
     def get_pool_member(self, context, id):
         member_db = self._get_resource(context, models.MemberV2, id)
         return data_models.Member.from_sqlalchemy_model(member_db)
 
+    def get_pool_member_as_api_dict(self, context, id):
+        member_db = self._get_resource(context, models.MemberV2, id)
+        return member_db.to_api_dict
+
     def delete_member(self, context, id):
         with context.session.begin(subtransactions=True):
             member_db = self._get_resource(context, models.MemberV2, id)
@@ -742,11 +813,12 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
         with context.session.begin(subtransactions=True):
             hm_db = self.create_healthmonitor(context, healthmonitor)
             pool = self.get_pool(context, pool_id)
-            # do not want listener, members, healthmonitor or loadbalancer
-            # in dict
+            # do not want listener, members, l7_policies, healthmonitor
+            # or loadbalancer in dict
             pool_dict = pool.to_dict(listeners=False, members=False,
-                                     healthmonitor=False, loadbalancer=False,
-                                     listener=False, loadbalancer_id=False)
+                                     l7_policies=False, healthmonitor=False,
+                                     loadbalancer=False, listener=False,
+                                     loadbalancer_id=False)
             pool_dict['healthmonitor_id'] = hm_db.id
             self.update_pool(context, pool_id, pool_dict)
             hm_db = self._get_resource(context, models.HealthMonitorV2,
@@ -780,6 +852,10 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
         hm_db = self._get_resource(context, models.HealthMonitorV2, id)
         return data_models.HealthMonitor.from_sqlalchemy_model(hm_db)
 
+    def get_healthmonitor_as_api_dict(self, context, id):
+        hm_db = self._get_resource(context, models.HealthMonitorV2, id)
+        return hm_db.to_api_dict
+
     def get_healthmonitors(self, context, filters=None):
         filters = filters or {}
         hm_dbs = self._get_resources(context, models.HealthMonitorV2,
@@ -787,6 +863,16 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
         return [data_models.HealthMonitor.from_sqlalchemy_model(hm_db)
                 for hm_db in hm_dbs]
 
+    def get_healthmonitors_as_api_dict(self, context, filters=None):
+        options = (
+            subqueryload(models.HealthMonitorV2.pool)
+        )
+        filters = filters or {}
+        hm_dbs = self._get_resources(context, models.HealthMonitorV2,
+                                     filters=filters, options=options)
+        return [hm_db.to_api_dict
+                for hm_db in hm_dbs]
+
     def update_loadbalancer_stats(self, context, loadbalancer_id, stats_data):
         stats_data = stats_data or {}
         with context.session.begin(subtransactions=True):
@@ -853,7 +939,11 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
                     context, models.Listener, listener_id)
                 l7policy_db = listener_db.l7_policies.pop(
                     l7policy_db.position - 1)
-
+                # NOTE(ihrachys) create a new policy model because the one just
+                # popped off the list became expired thanks to neutron db code
+                # that automatically expires relationships when foreign keys
+                # become obsolete
+                l7policy_db = models.L7Policy(**dict(l7policy_db))
                 l7policy_db.update(l7policy)
                 listener_db.l7_policies.insert(l7policy['position'] - 1,
                                                l7policy_db)
@@ -874,12 +964,25 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
         l7policy_db = self._get_resource(context, models.L7Policy, id)
         return data_models.L7Policy.from_sqlalchemy_model(l7policy_db)
 
+    def get_l7policy_as_api_dict(self, context, id):
+        l7policy_db = self._get_resource(context, models.L7Policy, id)
+        return l7policy_db.to_api_dict
+
     def get_l7policies(self, context, filters=None):
         l7policy_dbs = self._get_resources(context, models.L7Policy,
                                            filters=filters)
         return [data_models.L7Policy.from_sqlalchemy_model(l7policy_db)
                 for l7policy_db in l7policy_dbs]
 
+    def get_l7policies_as_api_dict(self, context, filters=None):
+        options = (
+            subqueryload(models.L7Policy.rules)
+        )
+        l7policy_dbs = self._get_resources(context, models.L7Policy,
+                                           filters=filters, options=options)
+        return [l7policy_db.to_api_dict
+                for l7policy_db in l7policy_dbs]
+
     def create_l7policy_rule(self, context, rule, l7policy_id):
         with context.session.begin(subtransactions=True):
             if not self._resource_exists(context, models.L7Policy,
@@ -929,6 +1032,13 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
                 l7policy_id=l7policy_id, rule_id=id)
         return data_models.L7Rule.from_sqlalchemy_model(rule_db)
 
+    def get_l7policy_rule_as_api_dict(self, context, id, l7policy_id):
+        rule_db = self._get_resource(context, models.L7Rule, id)
+        if rule_db.l7policy_id != l7policy_id:
+            raise l7.RuleNotFoundForL7Policy(
+                l7policy_id=l7policy_id, rule_id=id)
+        return rule_db.to_api_dict
+
     def get_l7policy_rules(self, context, l7policy_id, filters=None):
         if filters:
             filters.update(filters)
@@ -939,11 +1049,24 @@ class LoadBalancerPluginDbv2(base_db.CommonDbMixin,
         return [data_models.L7Rule.from_sqlalchemy_model(rule_db)
                 for rule_db in rule_dbs]
 
+    def get_l7policy_rules_as_api_dict(
+            self, context, l7policy_id, filters=None):
+        options = (
+            subqueryload(models.L7Rule.policy)
+        )
+        if filters:
+            filters.update(filters)
+        else:
+            filters = {'l7policy_id': [l7policy_id]}
+        rule_dbs = self._get_resources(context, models.L7Rule,
+                                       filters=filters, options=options)
+        return [rule_db.to_api_dict
+                for rule_db in rule_dbs]
 
-def _prevent_lbaasv2_port_delete_callback(resource, event, trigger, **kwargs):
-    context = kwargs['context']
-    port_id = kwargs['port_id']
-    port_check = kwargs['port_check']
+
+def _prevent_lbaasv2_port_delete_callback(resource, event,
+                                          trigger, payload=None):
     lbaasv2plugin = directory.get_plugin(pg_const.LOADBALANCERV2)
-    if lbaasv2plugin and port_check:
-        lbaasv2plugin.db.prevent_lbaasv2_port_deletion(context, port_id)
+    if lbaasv2plugin and payload.metadata['port_check']:
+        lbaasv2plugin.db.prevent_lbaasv2_port_deletion(
+            payload.context, payload.resource_id)
