@@ -48,13 +48,13 @@ fi
 
 function generate_testr_results {
     # Give job user rights to access tox logs
-    sudo -H -u "$owner" chmod o+rw .
-    sudo -H -u "$owner" chmod o+rw -R .testrepository
-    if [ -f ".testrepository/0" ] ; then
-        .tox/"$testenv"/bin/subunit-1to2 < .testrepository/0 > ./testrepository.subunit
-        $SCRIPTS_DIR/subunit2html ./testrepository.subunit testr_results.html
-        gzip -9 ./testrepository.subunit
-        gzip -9 ./testr_results.html
+    if [ -f ".stestr/0" ] ; then
+        sudo -H -u "$owner" chmod o+rw .
+        sudo -H -u "$owner" chmod o+rw -R .stestr
+        .tox/"$testenv"/bin/subunit-1to2 < .stestr/0 > ./stestr.subunit
+        $SCRIPTS_DIR/subunit2html ./stestr.subunit stestr_results.html
+        gzip -9 ./stestr.subunit
+        gzip -9 ./stestr_results.html
         sudo mv ./*.gz /opt/stack/logs/
     fi
 }
