@@ -15,6 +15,7 @@
 
 import time
 
+from neutron_lib import constants as n_constants
 from oslo_log import log as logging
 from tempest.api.network import base
 from tempest import config
@@ -124,6 +125,29 @@ class BaseTestCase(base.BaseNetworkTest):
             test_utils.call_and_ignore_notfound_exc(
                 cls._delete_load_balancer, lb_id)
 
+        # Wait for all LBs to be fully deleted
+        for lb_id in cls._lbs_to_delete:
+            cls._wait_for_load_balancer_status(
+                lb_id, provisioning_status='DELETED', operating_status=None,
+                delete=True)
+
+        # Wait for straggling port deletes to finish (neutron will sometimes
+        # claim they are deleted before they are totally finished deleting)
+        remaining_ports = cls.ports_client.list_ports(
+            network_id=cls.network['id']).get('ports')
+        tries = 1
+        while remaining_ports and tries <= 10:
+            LOG.warning("Ports still exist on network %s (try #%s): %s",
+                        cls.network['id'], tries, remaining_ports)
+            time.sleep(10)
+            remaining_ports = cls.ports_client.list_ports(
+                network_id=cls.network['id']).get('ports')
+            tries += 1
+
+        if remaining_ports:
+            LOG.error("Failed to remove all remaining ports on network %s. "
+                      "Subnet deletion will probably fail.", cls.network['id'])
+
         super(BaseTestCase, cls).resource_cleanup()
 
     @classmethod
@@ -146,8 +170,6 @@ class BaseTestCase(base.BaseNetworkTest):
             cls._wait_for_load_balancer_status(lb.get('id'))
 
         cls._lbs_to_delete.append(lb.get('id'))
-        port = cls.ports_client.show_port(lb['vip_port_id'])
-        cls.ports.append(port['port'])
         return lb
 
     @classmethod
@@ -181,6 +203,10 @@ class BaseTestCase(base.BaseNetworkTest):
         timeout = 600
         end_time = time.time() + timeout
         lb = {}
+        # When running with no-op drivers there is no actual health to
+        # observe, so disable operating_status checks when running no-op.
+        if CONF.lbaas.test_with_noop:
+            operating_status = None
         while time.time() < end_time:
             try:
                 lb = cls.load_balancers_client.get_load_balancer(
@@ -194,9 +220,11 @@ class BaseTestCase(base.BaseNetworkTest):
                             _("loadbalancer {lb_id} not"
                               " found").format(
                                   lb_id=load_balancer_id))
-                if (lb.get('provisioning_status') == provisioning_status and
-                        lb.get('operating_status') == operating_status):
-                    break
+                if lb.get('provisioning_status') == provisioning_status:
+                    if operating_status is None:
+                        break
+                    elif lb.get('operating_status') == operating_status:
+                        break
                 time.sleep(interval_time)
             except exceptions.NotFound:
                 # if wait is for delete operation do break
@@ -311,6 +339,8 @@ class BaseTestCase(base.BaseNetworkTest):
         if wait:
             self._wait_for_load_balancer_status(
                 self.load_balancer.get('id'))
+            health_monitor = self.health_monitors_client.get_health_monitor(
+                health_monitor_id)
         return health_monitor
 
     @classmethod
@@ -343,7 +373,12 @@ class BaseTestCase(base.BaseNetworkTest):
         statuses = cls.load_balancers_client.get_load_balancer_status_tree(
             load_balancer_id=load_balancer_id)
         load_balancer = statuses['loadbalancer']
-        assert 'ONLINE' == load_balancer['operating_status']
+
+        # When running with no-op drivers there is no actual health to
+        # observe, so disable operating_status checks when running no-op.
+        if not CONF.lbaas.test_with_noop:
+            assert 'ONLINE' == load_balancer['operating_status']
+
         assert 'ACTIVE' == load_balancer['provisioning_status']
 
         if listener_ids:
@@ -365,13 +400,16 @@ class BaseTestCase(base.BaseNetworkTest):
     @classmethod
     def _check_status_tree_thing(cls, actual_thing_ids, status_tree_things):
         found_things = 0
-        status_tree_things = status_tree_things
         assert len(actual_thing_ids) == len(status_tree_things)
         for actual_thing_id in actual_thing_ids:
             for status_tree_thing in status_tree_things:
                 if status_tree_thing['id'] == actual_thing_id:
-                    assert 'ONLINE' == (
-                        status_tree_thing['operating_status'])
+                    # When running with no-op drivers there is no actual
+                    # health to observe, so disable operating_status checks
+                    # when running no-op.
+                    if not CONF.lbaas.test_with_noop:
+                        assert 'ONLINE' == (
+                            status_tree_thing['operating_status'])
                     assert 'ACTIVE' == (
                         status_tree_thing['provisioning_status'])
                     found_things += 1
@@ -385,6 +423,17 @@ class BaseTestCase(base.BaseNetworkTest):
         )
         return name
 
+    def _test_provisioning_status_if_exists(self, created_obj, get_obj):
+        prov_status_old = created_obj.pop('provisioning_status', None)
+        prov_status_new = get_obj.pop('provisioning_status', None)
+
+        if prov_status_old:
+            self.assertIn('PENDING_', prov_status_old)
+        if prov_status_new:
+            self.assertEqual(n_constants.ACTIVE, prov_status_new)
+        if not created_obj.get('updated_at') and get_obj.get('updated_at'):
+            get_obj['updated_at'] = None
+
 
 class BaseAdminTestCase(BaseTestCase):
 
