@@ -15,11 +15,13 @@
 
 import collections
 
+from neutron_lbaas.services.loadbalancer import constants
+
 RET_PERSISTENCE = {
     'type': 'HTTP_COOKIE',
     'cookie_name': 'HTTP_COOKIE'}
 
-HASHSEED_ORDERED_CODES = list({'404', '405', '500'})
+HASHSEED_ORDERED_CODES = ['404', '405', '500']
 PIPED_CODES = '|'.join(HASHSEED_ORDERED_CODES)
 
 RET_MONITOR = {
@@ -76,7 +78,7 @@ RET_LISTENER = {
 RET_LISTENER_TLS = {
     'id': 'sample_listener_id_1',
     'protocol_port': '443',
-    'protocol_mode': 'HTTP',
+    'protocol_mode': 'http',
     'protocol': 'TERMINATED_HTTPS',
     'default_pool': RET_POOL,
     'connection_limit': 98,
@@ -99,21 +101,21 @@ RET_LISTENER_TLS_SNI = {
     'sni_containers': [RET_SNI_CONT_1, RET_SNI_CONT_2]}
 
 RET_LB = {
-    'name': 'test-lb',
+    'id': 'sample_loadbalancer_id_1',
     'vip_address': '10.0.0.2',
     'listeners': [RET_LISTENER],
     'connection_limit': RET_LISTENER['connection_limit'],
     'pools': [RET_POOL]}
 
 RET_LB_TLS = {
-    'name': 'test-lb',
+    'id': 'sample_loadbalancer_id_1',
     'vip_address': '10.0.0.2',
     'listeners': [RET_LISTENER_TLS],
     'connection_limit': RET_LISTENER_TLS['connection_limit'],
     'pools': [RET_POOL]}
 
 RET_LB_TLS_SNI = {
-    'name': 'test-lb',
+    'id': 'sample_loadbalancer_id_1',
     'vip_address': '10.0.0.2',
     'listeners': [RET_LISTENER_TLS_SNI],
     'connection_limit': RET_LISTENER_TLS_SNI['connection_limit'],
@@ -124,11 +126,10 @@ def sample_loadbalancer_tuple(proto=None, monitor=True, persistence=True,
                               persistence_type=None, tls=False, sni=False):
     proto = 'HTTP' if proto is None else proto
     in_lb = collections.namedtuple(
-        'loadbalancer', 'id, name, vip_address, protocol, vip_port, '
+        'loadbalancer', 'id, vip_address, protocol, vip_port, '
                         'listeners, pools')
     return in_lb(
         id='sample_loadbalancer_id_1',
-        name='test-lb',
         vip_address='10.0.0.2',
         protocol=proto,
         vip_port=sample_vip_port_tuple(),
@@ -268,26 +269,54 @@ def sample_health_monitor_tuple(proto='HTTP', admin_state=True):
 
     return monitor(id='sample_monitor_id_1', type=proto, delay=30,
                    timeout=31, max_retries=3, http_method='GET',
-                   url_path='/index.html', expected_codes='500, 405, 404',
+                   url_path='/index.html', expected_codes='404, 405, 500',
                    admin_state_up=admin_state)
 
 
-def sample_base_expected_config(backend, frontend=None):
+def sample_base_expected_config(backend, frontend=None,
+                                fe_proto=constants.PROTOCOL_HTTP):
     if frontend is None:
-        frontend = ("frontend sample_listener_id_1\n"
-                    "    option tcplog\n"
-                    "    maxconn 98\n"
-                    "    option forwardfor\n"
-                    "    bind 10.0.0.2:80\n"
-                    "    mode http\n"
-                    "    default_backend sample_pool_id_1\n\n")
-    return ("# Configuration for test-lb\n"
+        tcp_frontend = ("frontend sample_listener_id_1\n"
+                        "    option tcplog\n"
+                        "    maxconn 98\n"
+                        "    bind 10.0.0.2:80\n"
+                        "    mode tcp\n"
+                        "    default_backend sample_pool_id_1\n\n")
+        http_frontend = ("frontend sample_listener_id_1\n"
+                         "    option httplog\n"
+                         "    maxconn 98\n"
+                         "    option forwardfor\n"
+                         "    bind 10.0.0.2:80\n"
+                         "    mode http\n"
+                         "    default_backend sample_pool_id_1\n\n")
+        https_frontend = ("frontend sample_listener_id_1\n"
+                          "    option tcplog\n"
+                          "    maxconn 98\n"
+                          "    bind 10.0.0.2:443\n"
+                          "    mode tcp\n"
+                          "    default_backend sample_pool_id_1\n\n")
+        https_tls_frontend = ("frontend sample_listener_id_1\n"
+                              "    option httplog\n"
+                              "    redirect scheme https if !{ ssl_fc }\n"
+                              "    maxconn 98\n"
+                              "    option forwardfor\n"
+                              "    bind 10.0.0.2:443\n"
+                              "    mode http\n"
+                              "    default_backend sample_pool_id_1\n\n")
+        fe_mapper = {
+            constants.PROTOCOL_TCP: tcp_frontend,
+            constants.PROTOCOL_HTTP: http_frontend,
+            constants.PROTOCOL_HTTPS: https_frontend,
+            constants.PROTOCOL_TERMINATED_HTTPS: https_tls_frontend
+        }
+        frontend = fe_mapper[fe_proto]
+    return ("# Configuration for sample_loadbalancer_id_1\n"
             "global\n"
             "    daemon\n"
             "    user nobody\n"
             "    group nogroup\n"
-            "    log /dev/log local0\n"
-            "    log /dev/log local1 notice\n"
+            "    log /dev/log local0 debug alert\n"
+            "    log /dev/log local1 notice alert\n"
             "    maxconn 98\n"
             "    stats socket /sock_path mode 0666 level user\n\n"
             "defaults\n"
